@@ -30,7 +30,8 @@ use Scalar::Util qw(weaken blessed);
 use Evented::Object::EventFire;
 use Evented::Object::Collection;
 
-our $VERSION = '5.4';
+# always using 2 decimals now. change other packages too.
+our $VERSION = '5.41';
 
 # create a new evented object.
 sub new {
@@ -66,7 +67,8 @@ sub register_callback {
     if (my $before = defined $opts{before} or defined $opts{after}) {
         my $add   = $before ? 1 : -1;
         my $res   = $eo->_get_callback_named($event_name, $opts{before} // $opts{after});
-        $priority = $res->[0] + $add if defined $res;
+        $priority = defined $res ? $res->[0] + $add : 'nan';
+        # nan priority indicates it should be determined at a later time.
     }
     
     # add the callback.
@@ -293,14 +295,6 @@ sub delete_listener {
 ### CLASS FUNCTIONS ###
 #######################
 
-# export a subroutine.
-# export_code('My::Package', 'my_sub', \&_my_sub)
-sub export_code {
-    my ($package, $sub_name, $code) = @_;
-    no strict 'refs';
-    *{"${package}::$sub_name"} = $code;
-}
-
 # safely fire an event.
 sub safe_fire {
     my $obj = shift;
@@ -404,7 +398,7 @@ sub _get_callbacks {
             foreach my $priority (keys %{ $obj->{$events}{$listener_event_name} }) {
 
                 # create a group reference.
-                my $group = [ $eo, $listener_event_name, \@args]; # XXX: $eo or $obj?
+                my $group = [ $eo, $listener_event_name, \@args];
                 
                 # add each callback.
                 foreach my $cb (@{ $obj->{$events}{$listener_event_name}{$priority} }) {
@@ -508,7 +502,7 @@ and fire objects and their methods.
  
  use warnings;
  use strict;
- use feature 'say';
+ use 5.010;
  use parent 'Evented::Object';
  
  use Evented::Object;
@@ -570,8 +564,9 @@ In some other package...
  
 =head1 DESCRIPTION
 
-B<I honestly doubt your objects have ever been this evented in your entire life.> This concept is so incredible that
-we're using a noun as a verb without being arrested by the grammar police.
+B<I honestly doubt your objects have ever been this evented in your entire life.> This
+concept is so incredible that we're using a noun as a verb without being arrested by the
+grammar police.
 
 Evented::Object started as a basic class for registering event handlers and firing events.
 After many improvements throughout several projects, Evented::Object has become far more
@@ -581,11 +576,6 @@ Evented::Object supplies an (obviously objective) interface to store and manage
 for events, fire events upon objects, and more. It provides several methods for
 convenience and simplicity.
 
-=head2 Introduction
-
-First and foremost, the goal of Evented::Object is to make your objects more evented than
-ever before. Allow us to explain what exactly it means for an object to be evented.
-
 =head2 Naming confusion
 
 To clear some things up...
@@ -598,15 +588,19 @@ Evented::Object class. 'Fire object' refers to an object representing an event f
 
 =item *
 
-B<Evented::Object>: the class that provides methods for managing events.
+B<Evented::Object>: this class that provides methods for managing events.
 
 =item *
 
-B<Evented object>: C<$eo> - an object that uses Evented::Object for event management.
+B<Evented object>: C<$eo> - refers to an object that uses Evented::Object for event management.
 
 =item *
 
-B<Fire object>: C<$fire> - an object that represents an event fire.
+B<Fire object>: C<$fire> or C<$event> - an object that represents an event fire.
+
+=item *
+
+B<Collection>: C<$col> or C<$collection> - represents a group of callbacks about to be fired.
 
 =item *
 
@@ -647,6 +641,15 @@ more. Additionally, it provides an interface for modifying the evented object an
 modifying future event callbacks. Fire objects belong to the
 Evented::Object::EventFire class.
 
+Fire objects are specific to each firing. If you fire the same event twice in a row,
+the event object passed to the callbacks the first time will not be the same as the second
+time. Therefore, all modifications made by the fire object's methods apply only to
+the callbacks remaining in this particular fire. For example,
+C<$fire-E<gt>cancel($callback)> will only cancel the supplied callback once. The next
+time the event is fired, that cancelled callback will be called regardless.
+
+See L</"Fire object methods"> for more information.
+
 =head2 Listener objects
 
 Additional evented objects can be registered as "listeners."
@@ -699,35 +702,54 @@ callbacks, including those belonging to the evented object.
 
 =head2 Registering callbacks to classes
 
-Evented::Object 3.9 adds the ability to register event callbacks to a subclass of Evented::Object.
-The methods C<-E<gt>register_callback()>, C<-E<gt>delete_event()>, C<-E<gt>delete_callback>, etc. can be called in
-the form of C<MyClass-E<gt>method()>. Evented::Object will store these callbacks in a special hash hidden
-in the package's symbol table.  
+Evented::Object 3.9 adds the ability to register event callbacks to a subclass of
+Evented::Object. The methods C<-E<gt>register_callback()>, C<-E<gt>delete_event()>,
+C<-E<gt>delete_callback>, etc. can be called in the form of C<MyClass-E<gt>method()>.
+Evented::Object will store these callbacks in a special hash hidden in the package's
+symbol table.  
   
-Any object of this class will borrow these callbacks from the class. They will be incorporated into the callback collection as though they were registered directly on the object.
+Any object of this class will borrow these callbacks from the class. They will be
+incorporated into the callback collection as though they were registered directly on the
+object.
 
 Note: Events cannot be fired on a class.
 
 =head3 Prioritizing
 
-When firing an event, any callbacks on the class will sorted by priority just as if they were registered on the object. Whether registered on the class or the object, a callback with a
-higher priority will be called before one of a lower priority.
+When firing an event, any callbacks on the class will sorted by priority just as if they
+were registered on the object. Whether registered on the class or the object, a callback
+with a higher priority will be called before one of a lower priority.
 
 =head3 Subclassing
 
 If an evented object is blessed to a subclass of a class with callbacks registered to it,
-the object will NOT inherit the callbacks associated with the parent class. Callbacks registered
-to classes ONLY apply to objects directly blessed to the class.
+the object will NOT inherit the callbacks associated with the parent class. Callbacks
+registered to classes ONLY apply to objects directly blessed to the class.
 
 =head2 Class monitors
 
-Evented::Object 4.0 introduces a "class monitor" feature. This allows an evented object to be registered
-as a "monitor" of a specific class/package. Any event callbacks that are added from that class to any
-evented object of any type will trigger an event on the monitor object - in other words, the `caller` of
-`->register_callback()`, regardless of the object.
+Evented::Object 4.0 introduces a "class monitor" feature. This allows an evented object to
+be registered as a "monitor" of a specific class/package. Any event callbacks that are
+added from that class to any evented object of any type will trigger an event on the
+monitor object - in other words, the `caller` of `->register_callback()`, regardless of
+the object.
 
-An example scenario of when this might be useful is an evented object for debugging all events being
-registered by a certain package. It would log all of them, making it easier to find a problem.
+An example scenario of when this might be useful is an evented object for debugging all
+events being registered by a certain package. It would log all of them, making it easier
+to find a problem.
+
+=head2 Collections
+
+Sometimes it is useful to prepare an event fire before actually calling it. The group
+of callbacks that are about to be called are represented by a collection object.
+Collections are returned by the 'prepare' methods.
+
+Collections are especially useful for firing events with special options. This usually
+looks something like:
+
+ $eo->prepare(event_name => @args)->fire(some_fire_option => $value);
+
+See L</"Collection methods"> for more information.
 
 =head1 COMPATIBILITY
 
@@ -747,7 +769,7 @@ versions, the evented object was always the first argument of all events, until
 Evented::Object 0.6 added the ability to pass a parameter to C<-E<gt>attach_event()> that
 would tell Evented::Object to omit the object from the callback's argument list.
 
-=head2 Introduction of fire objects 1.8+
+=head2 Introduction of fire info 1.8+
 
 The Evented::Object series 1.8+ passes a hash reference C<$fire> instead of the
 Evented::Object as the first argument. C<$fire> contains information that was formerly
@@ -768,7 +790,7 @@ Evented::Object until at least the late 2.* series.
 Version 2.0 breaks things even more because C<-E<gt>on()> is now an alias for
 C<-E<gt>register_callback()> rather than the former deprecated C<-E<gt>attach_event()>.
 
-=head2 Introduction of event methods 2.2+
+=head2 Introduction of fire objects 2.2+
 
 Version 2.2+ introduces a new class, Evented::Object::EventFire, which provides several
 methods for fire objects. These methods such as C<$fire-E<gt>return> and
@@ -788,7 +810,7 @@ In order to correspond with other 'Evented' packages, EventedObject was renamed
 Evented::Object. All packages making use of EventedObject will need to be modified to use
 Evented::Object instead. This change was made pre-CPAN.
 
-=head1 EVENTED OBJECT METHODS
+=head1 Evented object methods
 
 The Evented::Object package provides several convenient methods for managing an
 event-driven object.
@@ -804,10 +826,9 @@ Evented::Object.
 
 =head2 $eo->register_callback($event_name => \&callback, %options)
 
-Intended to be a replacement for the former C<-E<gt>attach_event()>. Attaches an event
-callback the object. When the specified event is fired, each of the callbacks registered
-using this method will be called by descending priority order (higher priority numbers are
-called first.)
+Attaches an event callback the object. When the specified event is fired, each of the
+callbacks registered using this method will be called by descending priority order
+(numerically higher priority numbers are called first.)
 
  $eo->register_callback(myEvent => sub {
      ...
@@ -862,24 +883,12 @@ fired.
 
 =item *
 
+B<with_eo>: if true, the evented object will prepended to the argument list.
+
+=item *
+
 B<no_fire_obj>: if true, the fire object will not be prepended to the argument list.
 
-=item *
-
-B<with_evented_obj>: if true, the evented object will prepended to the argument list.
-
-=item *
-
-B<no_obj>: I<Deprecated>. Use C<no_fire_obj> instead.
-
-=item *
-
-B<eo_obj>: I<Deprecated>. Use C<with_evented_obj> instead.
-
-=item *
-
-B<with_obj>: I<Deprecated>. Use C<with_evented_obj> instead.
-
 =back
 
 Note: the order of objects will always be C<$eo>, C<$fire>, C<@args>, regardless of
@@ -969,6 +978,32 @@ B<arguments>: I<optional>, list of arguments to pass to event callbacks.
 
 =back
 
+=head2 $eo->fire_once($event_name => @arguments)
+
+Fires the specified event, calling each callback that was registered with
+C<-E<gt>register_callback()> in descending order of their priorities.
+
+Then, all callbacks for the event are deleted. This method is useful for situations where
+an event will never be fired more than once.
+
+ $eo->fire_once('some_event');
+ $eo->fire_event(some_event => $some_argument, $some_other_argument);
+ # the second does nothing because the first deleted the callbacks
+
+B<Parameters>
+
+=over 4
+
+=item *
+
+B<event_name>: the name of the event being fired.
+
+=item *
+
+B<arguments>: I<optional>, list of arguments to pass to event callbacks.
+
+=back
+
 =head2 $eo->add_listener($other_eo, $prefix)
 
 Makes the passed evented object a listener of this evented object. See the "listener
@@ -990,6 +1025,11 @@ B<prefix>: a string that event names will be prefixed with on the listener.
 
 =back
 
+=head2 $eo->fire_events_together(@events)
+
+Since Evented::Object 5.0, the C<fire_events_together()> function can be used as a method
+on evented objects. See the documentation for the function in L</"Procedural functions">.
+
 =head2 $eo->delete_listener($other_eo)
 
 Removes a listener of this evented object. See the "listener objects" section for more
@@ -1011,24 +1051,38 @@ B<prefix>: a string that event names will be prefixed with on the listener.
 
 =back
 
-=head2 $eo->on($event_name => \&callback, %options)
+=head1 Preparation methods
 
-Alias for C<-E<gt>register_callback()>.
+Evented::Object 5.0 introduces a means by which callbacks can be prepared before being
+fired. This is most useful for firing events with special fire options.
 
-=head2 $eo->fire($event_name => @arguments)
+=head2 $eo->prepare_event(event_name => @arguments)
 
-Alias for C<-E<gt>fire_event()>.
+Prepares a single event for firing. Returns a collection object representing the callbacks
+for the event.
 
-=head2 $eo->del(...)
+ # an example using the fire option return_check.
+ $eo->prepare_event(some_event => @arguments)->fire('return_check');
 
-B<Deprecated>. Alias for C<-E<gt>delete_event()>.
-Do not use this. It is likely to removed in the near future.
+=head2 $eo->prepare_together(@events)
 
-=head2 $eo->attach_event(...)
+The preparatory method equivalent to C<-E<gt>fire_events_together>. 
 
-B<Removed> in version 2.9. Use C<-E<gt>register_callback()> instead.
+=head2 $eo->prepare(...)
 
-=head1 EVENTED::OBJECT PROCEDURAL FUNCTIONS
+A smart method that uses the best guess between C<-E<gt>prepare_event> and
+C<-E<gt>prepare_together>.
+
+ # uses ->prepare_event()
+ $eo->prepare(some_event => @arguments);
+ 
+ # uses ->prepare_together()
+ $eo->prepare(
+    [ some_event => @arguments ],
+    [ some_other => @other_arg ]
+ );
+
+=head1 Procedural functions
 
 The Evented::Object package provides some functions for use. These functions typically are
 associated with more than one evented object or none at all.
@@ -1055,7 +1109,31 @@ C<[ $evented_object, event_name =E<gt> @arguments ]>
      [ $channel, user_joined         => $user           ],
      [ $user,    joined_channel      => $channel        ]
  );
+ 
+Since Evented::Object 5.0, C<-E<gt>fire_events_together> can be used as a method on any
+evented object.
 
+ $eo->fire_events_together(
+     [ some_event => @arguments ],
+     [ some_other => @other_arg ]
+ );
+ 
+The above example would formerly be achieved as:
+
+ Evented::Object::fire_events_together(
+     [ $eo, some_event => @arguments ],
+     [ $eo, some_other => @other_arg ]
+ );
+ 
+However, other evented objects may be specified even when this is used as a method.
+Basically, anywhere that an object is missing will fall back to the object on which
+the method was called.
+
+ $eo->fire_events_together(
+     [ $other_eo, some_event => @arguments ],
+     [            some_other => @other_arg ] # no object, falls back to $eo
+ );
+ 
 B<Parameters>
 
 =over 4
@@ -1082,14 +1160,10 @@ B<Parameters>
 
 B<eo>: the evented object.
 
-=back
-
 =item *
 
 B<event_name>: the name of the event. 
 
-=back
-
 =item *
 
 B<args>: the arguments for the event fire.
@@ -1148,47 +1222,67 @@ B<__some_eo__>: some arbitrary event object that is responding to that activity.
 
 =back
 
-=head2 export_code($package, $sub_name, $code)
+=head1 Collection methods
 
-Exports a code reference to the symbol table of the specified package name.
+L</Collections> are returned by the 'prepare' methods. They represent a group of callbacks
+that are about to be fired.
 
- my $code = sub { say 'Hello world!' };
- Evented::Object::export_code('MyPackage', 'hello', $code);
+=head2 $col->fire(@options)
 
+Fires the pending callbacks with the specified options, if any. If the callbacks have not
+yet been sorted, they are sorted before the event is fired.
+
+ $eo->prepare(some_event => @arguments)->fire('safe');
+ 
 B<Parameters>
 
 =over 4
 
 =item *
 
-B<package>: name of package.
-
-=item *
-
-B<sub_name>: name of desired symbol.
-
-=item *
-
-B<code>: code reference to export.
+B<@options>: I<optional>, a mixture of boolean and key:value options for the event fire.
 
 =back
 
-=head1 FIRE OBJECT METHODS
+B<@options>
 
-Fire objects are passed to all callbacks of an Evented::Object (unless the silent
+=over 4
+
+=item *
+
+B<caller>: I<requires value>, use an alternate C<[caller 1]> value for the event fire.
+This is typically only used internally.
+
+=item *
+
+B<return_check>: I<boolean>, if true, the event will yield that it was stopped if any
+of the callbacks return a false value. Note however that if one callbacks returns false,
+the rest will still be called. The fire object will only yield stopped status after all
+callbacks have been called and any number of them returned false.
+
+=item *
+
+B<safe>: I<boolean>, wrap all callback calls in eval for safety. if any of them fail,
+the event will be stopped at that point with the error.
+
+=item *
+
+B<fail_continue>: I<boolean>, if C<safe> above is enabled, this tells the fire to continue
+even if one of the callbacks fails. This could be dangerous if any of the callbacks
+expected a previous callback to be done when it actually failed.
+
+=back
+
+=head2 $col->sort_callbacks
+
+Sorts the callbacks according to C<priority>, C<before>, and C<after> options.
+
+=head1 Fire object methods
+
+L</"Fire objects"> are passed to all callbacks of an Evented::Object (unless the silent
 parameter was specified.) Fire objects contain information about the event itself,
 the callback, the caller of the event, event data, and more.
 
-Fire objects replace the former values stored within the Evented::Object itself.
-This new method promotes asynchronous event firing.
-
-Fire objects are specific to each firing. If you fire the same event twice in a row,
-the event object passed to the callbacks the first time will not be the same as the second
-time. Therefore, all modifications made by the fire object's methods apply only to
-the callbacks remaining in this particular fire. For example,
-C<$fire-E<gt>cancel($callback)> will only cancel the supplied callback once. The next
-time the event is fired, that cancelled callback will be called regardless.
-
 =head2 $fire->object
 
 Returns the evented object.
@@ -1204,10 +1298,13 @@ to determine from where the event was fired.
  my @caller = $fire->caller;
  say "Package $caller[0] line $caller[2] called event $name";
 
-=head2 $fire->stop
+=head2 $fire->stop($reason)
 
 Cancels all remaining callbacks. This stops the rest of the event firing. After a callback
-calls $fire->stop, it is stored as C<$fire-E<gt>stopper>.
+calls $fire->stop, the name of that callback is stored as C<$fire-E<gt>stopper>.
+
+If the event has already been stopped, this method returns the reason for which the
+fire was stopped or "unspecified" if no reason was given.
 
  # ignore messages from trolls
  if ($user eq 'noah') {
@@ -1216,6 +1313,14 @@ calls $fire->stop, it is stored as C<$fire-E<gt>stopper>.
      return $fire->stop;
  }
 
+=over 4
+
+=item *
+
+B<reason>: I<optional>, the reason for stopping the event fire.
+
+=back
+
 =head2 $fire->stopper
 
 Returns the callback which called C<$fire-E<gt>stop>.
@@ -1252,10 +1357,10 @@ If no argument is supplied, returns the number of callbacks pending to be called
 excluding the current one. If a callback  argument is supplied, returns whether that
 particular callback is pending for being called.
  
- say $fire->pending, 'callbacks are left.';
+ say $fire->pending, ' callbacks are left.';
  
  if ($fire->pending('some.callback')) {
-     say 'some.callback will be called soon.';
+     say 'some.callback will be called soon (unless it gets canceled)';
  }
 
 B<Parameters>
@@ -1350,10 +1455,38 @@ Returns the data supplied to the callback when it was registered, if any.
 
  say 'my data is ', $fire->callback_data;
 
+=head1 Aliases
+
+A number of aliases exist for convenience, but some of the names are rather broad. For
+that reason, they are only recommended for use when you are sure that other subclassing
+will not interfere.
+
+=head2 $eo->on(...)
+
+Alias for C<$eo-E<gt>register_callback()>.
+
+=head2 $eo->del(...)
+
+If one argument provided, alias for C<$eo-E<gt>delete_event>.
+
+If two arguments provided, alias for C<$eo-E<gt>delete_callback>.
+
+=head2 $eo->fire(...)
+
+Alias for C<$eo-E<gt>fire_event()>.
+
+=head2 $eo->register_event(...)
+
+Alias for C<$eo-E<gt>register_callback()>.
+
+=head2 $eo->register_events(...)
+
+Alias for C<$eo-E<gt>register_callbacks()>.
+
 =head2 $fire->eo
 
-Alias for C<-E<gt>object()>.
- 
+Alias for C<$fire-E<gt>object()>.
+
 =head1 AUTHOR
 
 L<Mitchell Cooper|https://github.com/cooper> <cooper@cpan.org>
@@ -1364,15 +1497,15 @@ Copyright E<copy> 2011-2013. Released under BSD license.
 
 =item *
 
-B<IRC channel>: L<irc.notroll.net #k|irc://irc.notroll.net/k>
+B<IRC>: L<irc.notroll.net #k|irc://irc.notroll.net/k>
 
 =item *
 
-B<Email>: cooper@cpan.org
+B<Email>: L<cooper@cpan.org|mailto:cooper@cpan.org>
 
 =item *
 
-B<PAUSE/CPAN>: L<COOPER|http://search.cpan.org/~cooper/>
+B<CPAN>: L<COOPER|http://search.cpan.org/~cooper/>
 
 =item *
 
@@ -1382,4 +1515,4 @@ B<GitHub>: L<cooper|https://github.com/cooper>
 
 Comments, complaints, and recommendations are accepted. IRC is my preferred communication
 medium. Bugs may be reported on
-L<RT|https://rt.cpan.org/Public/Dist/Display.html?Name=Evented-Object>.
+L<RT|https://rt.cpan.org/Public/Dist/Display.html?Name=Evented-Object>.
\ No newline at end of file
